@@ -2,9 +2,10 @@ use std::borrow::Cow;
 
 use crate::parser::types::Field;
 use crate::resolver_utils::resolve_list;
-use crate::{registry, ContextSelectionSet, OutputType, Positioned, Type, Value};
+use crate::{registry, ContextSelectionSet, OutputType, Positioned, ServerResult, Value};
 
-impl<'a, T: Type + 'a> Type for &'a [T] {
+#[async_trait::async_trait]
+impl<'a, T: OutputType + 'a> OutputType for &'a [T] {
     fn type_name() -> Cow<'static, str> {
         Cow::Owned(format!("[{}]", T::qualified_type_name()))
     }
@@ -17,11 +18,12 @@ impl<'a, T: Type + 'a> Type for &'a [T] {
         T::create_type_info(registry);
         Self::qualified_type_name()
     }
-}
 
-#[async_trait::async_trait]
-impl<T: OutputType> OutputType for &[T] {
-    async fn resolve(&self, ctx: &ContextSelectionSet<'_>, field: &Positioned<Field>) -> Value {
+    async fn resolve(
+        &self,
+        ctx: &ContextSelectionSet<'_>,
+        field: &Positioned<Field>,
+    ) -> ServerResult<Value> {
         resolve_list(ctx, field, self.iter(), Some(self.len())).await
     }
 }
