@@ -175,6 +175,8 @@ impl<T> MaybeUndefined<T> {
 }
 
 impl<T: InputType> InputType for MaybeUndefined<T> {
+    type RawValueType = T::RawValueType;
+
     fn type_name() -> Cow<'static, str> {
         T::type_name()
     }
@@ -204,6 +206,14 @@ impl<T: InputType> InputType for MaybeUndefined<T> {
             _ => Value::Null,
         }
     }
+
+    fn as_raw_value(&self) -> Option<&Self::RawValueType> {
+        if let MaybeUndefined::Value(value) = self {
+            value.as_raw_value()
+        } else {
+            None
+        }
+    }
 }
 
 impl<T, E> MaybeUndefined<Result<T, E>> {
